@@ -1,5 +1,3 @@
-/* Created by Anjuta version 0.9.99 */
-/*	This file will not be overwritten */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -14,7 +12,6 @@
 #include "playground.h"
 #include <esd.h>
 #include "monkey.h"
-//#include "sound_fx.h"
 #include <math.h>
 #include "ui-main.h"
 #include "gdk-canvas.h"
@@ -33,13 +30,10 @@ int main(int argc, char **argv)
 #endif
   
   gtk_init (&argc, &argv);
-
-  HACK_load_contents();
   
   ui_main = ui_main_get_instance();
 
   window = ui_main_get_window(ui_main);
-  //gtk_window_new(GTK_WINDOW_TOPLEVEL);
   
   gdk_rgb_init ();
 
